@@ -71,6 +71,7 @@ from nose.tools import assert_set_equal
 from nose.tools import assert_is_instance
 from nose import SkipTest
 
+from datalad import cfg as dl_cfg
 import datalad.utils as ut
 # TODO this must go
 from ..utils import *
@@ -81,7 +82,11 @@ from datalad.utils import (
 
 from ..cmd import Runner
 from .. import utils
-from ..support.exceptions import CommandNotAvailableError
+from ..support.exceptions import (
+    CommandError,
+    CommandNotAvailableError,
+)
+from ..support.external_versions import external_versions
 from ..support.vcr_ import *
 from ..support.keyring_ import MemoryKeyring
 from ..support.network import RI
@@ -99,6 +104,12 @@ _TEMP_PATHS_CLONES = set()
 # Additional indicators
 on_travis = bool(os.environ.get('TRAVIS', False))
 
+if external_versions["cmd:git"] >= "2.28":
+    # The specific value here doesn't matter, but it should not be the default
+    # from any Git version to test that we work with custom values.
+    DEFAULT_BRANCH = "dl-test-branch"  # Set by setup_package().
+else:
+    DEFAULT_BRANCH = "master"
 
 # additional shortcuts
 neq_ = assert_not_equal
@@ -154,17 +165,17 @@ def skip_if_no_network(func=None):
     check_not_generatorfunction(func)
 
     def check_and_raise():
-        if os.environ.get('DATALAD_TESTS_NONETWORK'):
+        if dl_cfg.get('datalad.tests.nonetwork'):
             raise SkipTest("Skipping since no network settings")
 
     if func:
         @wraps(func)
         @attr('network')
         @attr('skip_if_no_network')
-        def newfunc(*args, **kwargs):
+        def  _wrap_skip_if_no_network(*args, **kwargs):
             check_and_raise()
             return func(*args, **kwargs)
-        return newfunc
+        return  _wrap_skip_if_no_network
     else:
         check_and_raise()
 
@@ -181,10 +192,10 @@ def skip_if_on_windows(func=None):
     if func:
         @wraps(func)
         @attr('skip_if_on_windows')
-        def newfunc(*args, **kwargs):
+        def  _wrap_skip_if_on_windows(*args, **kwargs):
             check_and_raise()
             return func(*args, **kwargs)
-        return newfunc
+        return  _wrap_skip_if_on_windows
     else:
         check_and_raise()
 
@@ -204,10 +215,10 @@ def skip_if_root(func=None):
     if func:
         @wraps(func)
         @attr('skip_if_root')
-        def newfunc(*args, **kwargs):
+        def  _wrap_skip_if_root(*args, **kwargs):
             check_and_raise()
             return func(*args, **kwargs)
-        return newfunc
+        return  _wrap_skip_if_root
     else:
         check_and_raise()
 
@@ -233,7 +244,7 @@ def skip_if(func, cond=True, msg=None, method='raise'):
     check_not_generatorfunction(func)
 
     @wraps(func)
-    def newfunc(*args, **kwargs):
+    def  _wrap_skip_if(*args, **kwargs):
         if cond:
             if method == 'raise':
                 raise SkipTest(msg if msg else "condition was True")
@@ -241,7 +252,7 @@ def skip_if(func, cond=True, msg=None, method='raise'):
                 print(msg if msg else "condition was True")
                 return
         return func(*args, **kwargs)
-    return newfunc
+    return  _wrap_skip_if
 
 
 def skip_ssh(func):
@@ -253,13 +264,12 @@ def skip_ssh(func):
 
     @wraps(func)
     @attr('skip_ssh')
-    def newfunc(*args, **kwargs):
-        from datalad import cfg
-        test_ssh = cfg.get("datalad.tests.ssh", '')
+    def  _wrap_skip_ssh(*args, **kwargs):
+        test_ssh = dl_cfg.get("datalad.tests.ssh", '')
         if not test_ssh or test_ssh in ('0', 'false', 'no'):
             raise SkipTest("Run this test by setting DATALAD_TESTS_SSH")
         return func(*args, **kwargs)
-    return newfunc
+    return  _wrap_skip_ssh
 
 
 @optional_args
@@ -271,10 +281,9 @@ def skip_v6_or_later(func, method='raise'):
     installed git-annex.
     """
 
-    from datalad import cfg
     from datalad.support.annexrepo import AnnexRepo
 
-    version = cfg.obtain("datalad.repo.version")
+    version = dl_cfg.obtain("datalad.repo.version")
     info = AnnexRepo.check_repository_versions()
 
     @skip_if(version >= 6 or 5 not in info["supported"],
@@ -282,9 +291,9 @@ def skip_v6_or_later(func, method='raise'):
     @wraps(func)
     @attr('skip_v6_or_later')
     @attr('v6_or_later')
-    def newfunc(*args, **kwargs):
+    def  _wrap_skip_v6_or_later(*args, **kwargs):
         return func(*args, **kwargs)
-    return newfunc
+    return  _wrap_skip_v6_or_later
 
 
 #
@@ -533,7 +542,11 @@ def ok_file_has_content(path, content, strip=False, re_=False,
 def with_tree(t, tree=None, archives_leading_dir=True, delete=True, **tkwargs):
 
     @wraps(t)
-    def newfunc(*arg, **kw):
+    def  _wrap_with_tree(*arg, **kw):
+        if 'dir' not in tkwargs.keys():
+            # if not specified otherwise, respect datalad.tests.temp.dir config
+            # as this is a test helper
+            tkwargs['dir'] = dl_cfg.get("datalad.tests.temp.dir")
         tkwargs_ = get_tempfile_kwargs(tkwargs, prefix="tree", wrapped=t)
         d = tempfile.mkdtemp(**tkwargs_)
         create_tree(d, tree, archives_leading_dir=archives_leading_dir)
@@ -542,7 +555,7 @@ def with_tree(t, tree=None, archives_leading_dir=True, delete=True, **tkwargs):
         finally:
             if delete:
                 rmtemp(d)
-    return newfunc
+    return  _wrap_with_tree
 
 
 lgr = logging.getLogger('datalad.tests')
@@ -637,7 +650,7 @@ def serve_path_via_http(tfunc, *targs):
 
     @wraps(tfunc)
     @attr('serve_path_via_http')
-    def newfunc(*args, **kwargs):
+    def  _wrap_serve_path_via_http(*args, **kwargs):
 
         if targs:
             # if a path is passed into serve_path_via_http, then it's in targs
@@ -651,7 +664,7 @@ def serve_path_via_http(tfunc, *targs):
 
         with HTTPPath(path) as url:
             return tfunc(*(args + (path, url)), **kwargs)
-    return newfunc
+    return  _wrap_serve_path_via_http
 
 
 @optional_args
@@ -660,12 +673,12 @@ def with_memory_keyring(t):
     """
     @wraps(t)
     @attr('with_memory_keyring')
-    def newfunc(*args, **kwargs):
+    def  _wrap_with_memory_keyring(*args, **kwargs):
         keyring = MemoryKeyring()
         with patch("datalad.downloaders.credentials.keyring_", keyring):
             return t(*(args + (keyring,)), **kwargs)
 
-    return newfunc
+    return  _wrap_with_memory_keyring
 
 
 @optional_args
@@ -675,7 +688,7 @@ def without_http_proxy(tfunc):
 
     @wraps(tfunc)
     @attr('without_http_proxy')
-    def newfunc(*args, **kwargs):
+    def  _wrap_without_http_proxy(*args, **kwargs):
         # Such tests don't require real network so if http_proxy settings were
         # provided, we remove them from the env for the duration of this run
         env = os.environ.copy()
@@ -684,7 +697,7 @@ def without_http_proxy(tfunc):
         with patch.dict('os.environ', env, clear=True):
             return tfunc(*args, **kwargs)
 
-    return newfunc
+    return  _wrap_without_http_proxy
 
 
 @borrowkwargs(methodname=make_tempfile)
@@ -709,11 +722,15 @@ def with_tempfile(t, **tkwargs):
     """
 
     @wraps(t)
-    def newfunc(*arg, **kw):
+    def  _wrap_with_tempfile(*arg, **kw):
+        if 'dir' not in tkwargs.keys():
+            # if not specified otherwise, respect datalad.tests.temp.dir config
+            # as this is a test helper
+            tkwargs['dir'] = dl_cfg.get("datalad.tests.temp.dir")
         with make_tempfile(wrapped=t, **tkwargs) as filename:
             return t(*(arg + (filename,)), **kw)
 
-    return newfunc
+    return  _wrap_with_tempfile
 
 
 # ### ###
@@ -730,9 +747,8 @@ def probe_known_failure(func):
 
     @wraps(func)
     @attr('probe_known_failure')
-    def newfunc(*args, **kwargs):
-        from datalad import cfg
-        if cfg.obtain("datalad.tests.knownfailures.probe"):
+    def  _wrap_probe_known_failure(*args, **kwargs):
+        if dl_cfg.obtain("datalad.tests.knownfailures.probe"):
             assert_raises(Exception, func, *args, **kwargs)  # marked as known failure
             # Note: Since assert_raises lacks a `msg` argument, a comment
             # in the same line is helpful to determine what's going on whenever
@@ -740,7 +756,7 @@ def probe_known_failure(func):
             # wouldn't be very telling.
         else:
             return func(*args, **kwargs)
-    return newfunc
+    return  _wrap_probe_known_failure
 
 
 @optional_args
@@ -750,16 +766,15 @@ def skip_known_failure(func, method='raise'):
     Setting config datalad.tests.knownfailures.skip to a bool enables/disables
     skipping.
     """
-    from datalad import cfg
 
-    @skip_if(cond=cfg.obtain("datalad.tests.knownfailures.skip"),
+    @skip_if(cond=dl_cfg.obtain("datalad.tests.knownfailures.skip"),
              msg="Skip test known to fail",
              method=method)
     @wraps(func)
     @attr('skip_known_failure')
-    def newfunc(*args, **kwargs):
+    def  _wrap_skip_known_failure(*args, **kwargs):
         return func(*args, **kwargs)
-    return newfunc
+    return  _wrap_skip_known_failure
 
 
 def known_failure(func):
@@ -773,9 +788,9 @@ def known_failure(func):
     @probe_known_failure
     @wraps(func)
     @attr('known_failure')
-    def newfunc(*args, **kwargs):
+    def  _wrap_known_failure(*args, **kwargs):
         return func(*args, **kwargs)
-    return newfunc
+    return  _wrap_known_failure
 
 
 def known_failure_v6_or_later(func):
@@ -788,10 +803,9 @@ def known_failure_v6_or_later(func):
     installed git-annex.
     """
 
-    from datalad import cfg
     from datalad.support.annexrepo import AnnexRepo
 
-    version = cfg.obtain("datalad.repo.version")
+    version = dl_cfg.obtain("datalad.repo.version")
     info = AnnexRepo.check_repository_versions()
 
     if (version and version >= 6) or 5 not in info["supported"]:
@@ -879,6 +893,20 @@ def known_failure_githubci_win(func):
     return func
 
 
+def known_failure_githubci_osx(func):
+    """Test decorator for a known test failure on Github's macOS CI
+    """
+    if 'GITHUB_WORKFLOW' in os.environ and on_osx:
+        @known_failure
+        @wraps(func)
+        @attr('known_failure_githubci_osx')
+        @attr('githubci_osx')
+        def dm_func(*args, **kwargs):
+            return func(*args, **kwargs)
+        return dm_func
+    return func
+
+
 # ### ###
 # END known failure decorators
 # ### ###
@@ -894,7 +922,7 @@ def _get_resolved_flavors(flavors):
     if not isinstance(flavors_, list):
         flavors_ = [flavors_]
 
-    if os.environ.get('DATALAD_TESTS_NONETWORK'):
+    if dl_cfg.get('datalad.tests.nonetwork'):
         flavors_ = [x for x in flavors_ if not x.startswith('network')]
     return flavors_
 
@@ -903,7 +931,8 @@ def clone_url(url):
     # delay import of our code until needed for certain
     from ..cmd import Runner
     runner = Runner()
-    tdir = tempfile.mkdtemp(**get_tempfile_kwargs({}, prefix='clone_url'))
+    tdir = tempfile.mkdtemp(**get_tempfile_kwargs(
+        {'dir': dl_cfg.get("datalad.tests.temp.dir")}, prefix='clone_url'))
     _ = runner(["git", "clone", url, tdir], expect_stderr=True)
     if GitRepo(tdir).is_with_annex():
         AnnexRepo(tdir, init=True)
@@ -1015,7 +1044,7 @@ def with_testrepos(t, regex='.*', flavors='auto', skip=False, count=None):
     """
     @wraps(t)
     @attr('with_testrepos')
-    def newfunc(*arg, **kw):
+    def  _wrap_with_testrepos(*arg, **kw):
         # TODO: would need to either avoid this "decorator" approach for
         # parametric tests or again aggregate failures like sweepargs does
         flavors_ = _get_resolved_flavors(flavors)
@@ -1023,13 +1052,13 @@ def with_testrepos(t, regex='.*', flavors='auto', skip=False, count=None):
         testrepos_uris = _get_testrepos_uris(regex, flavors_)
         # we should always have at least one repo to test on, unless explicitly only
         # network was requested by we are running without networked tests
-        if not (os.environ.get('DATALAD_TESTS_NONETWORK') and flavors == ['network']):
+        if not (dl_cfg.get('datalad.tests.nonetwork') and flavors == ['network']):
             assert(testrepos_uris)
         else:
             if not testrepos_uris:
                 raise SkipTest("No non-networked repos to test on")
 
-        fake_dates = os.environ.get("DATALAD_FAKE__DATES")
+        fake_dates = dl_cfg.get("datalad.fake-dates")
         ntested = 0
         for uri in testrepos_uris:
             if count and ntested >= count:
@@ -1050,7 +1079,7 @@ def with_testrepos(t, regex='.*', flavors='auto', skip=False, count=None):
                     _TEMP_PATHS_CLONES.discard(uri)
                     rmtemp(uri)
                 pass  # might need to provide additional handling so, handle
-    return newfunc
+    return  _wrap_with_testrepos
 with_testrepos.__test__ = False
 
 
@@ -1071,7 +1100,16 @@ def with_sameas_remote(func, autoenabled=False):
     @skip_ssh
     @with_tempfile(mkdir=True)
     @with_tempfile(mkdir=True)
-    def newfunc(*args, **kwargs):
+    def  _wrap_with_sameas_remote(*args, **kwargs):
+        # With git-annex's 8.20200522-77-g1f2e2d15e, transferring from an rsync
+        # special remote hangs on Xenial. This is likely due to an interaction
+        # with an older rsync or openssh version. Use openssh as a rough
+        # indicator. See
+        # https://git-annex.branchable.com/bugs/Recent_hang_with_rsync_remote_with_older_systems___40__Xenial__44___Jessie__41__/
+        if external_versions['cmd:system-ssh'] < '7.4' and \
+           '8.20200522' < external_versions['cmd:annex'] < '8.20200720':
+            raise SkipTest("Test known to hang")
+
         sr_path, repo_path = args[-2:]
         fn_args = args[:-2]
         repo = AnnexRepo(repo_path)
@@ -1093,7 +1131,7 @@ def with_sameas_remote(func, autoenabled=False):
             # This should have --sameas support.
             raise
         return func(*(fn_args + (repo,)), **kwargs)
-    return newfunc
+    return  _wrap_with_sameas_remote
 
 
 @optional_args
@@ -1104,14 +1142,14 @@ def with_fake_cookies_db(func, cookies={}):
 
     @wraps(func)
     @attr('with_fake_cookies_db')
-    def newfunc(*args, **kwargs):
+    def  _wrap_with_fake_cookies_db(*args, **kwargs):
         try:
             orig_cookies_db = cookies_db._cookies_db
             cookies_db._cookies_db = cookies.copy()
             return func(*args, **kwargs)
         finally:
             cookies_db._cookies_db = orig_cookies_db
-    return newfunc
+    return  _wrap_with_fake_cookies_db
 
 
 @optional_args
@@ -1126,7 +1164,7 @@ def assert_cwd_unchanged(func, ok_to_chdir=False):
     """
 
     @wraps(func)
-    def newfunc(*args, **kwargs):
+    def  _wrap_assert_cwd_unchanged(*args, **kwargs):
         cwd_before = os.getcwd()
         pwd_before = getpwd()
         exc_info = None
@@ -1165,7 +1203,7 @@ def assert_cwd_unchanged(func, ok_to_chdir=False):
 
         return ret
 
-    return newfunc
+    return  _wrap_assert_cwd_unchanged
 
 
 @optional_args
@@ -1183,7 +1221,7 @@ def run_under_dir(func, newdir='.'):
     """
 
     @wraps(func)
-    def newfunc(*args, **kwargs):
+    def  _wrap_run_under_dir(*args, **kwargs):
         pwd_before = getpwd()
         try:
             chpwd(newdir)
@@ -1192,7 +1230,7 @@ def run_under_dir(func, newdir='.'):
             chpwd(pwd_before)
 
 
-    return newfunc
+    return  _wrap_run_under_dir
 
 
 def assert_re_in(regex, c, flags=0, match=True, msg=None):
@@ -1378,11 +1416,11 @@ def skip_httpretty_on_problematic_pythons(func):
     """
 
     @make_decorator(func)
-    def newfunc(*args, **kwargs):
+    def  _wrap_skip_httpretty_on_problematic_pythons(*args, **kwargs):
         if sys.version_info[:3] == (3, 4, 2):
             raise SkipTest("Known to cause trouble due to httpretty bug on this Python")
         return func(*args, **kwargs)
-    return newfunc
+    return  _wrap_skip_httpretty_on_problematic_pythons
 
 
 @optional_args
@@ -1390,31 +1428,22 @@ def with_parametric_batch(t):
     """Helper to run parametric test with possible combinations of batch and direct
     """
     @wraps(t)
-    def newfunc():
+    def  _wrap_with_parametric_batch():
         for batch in (False, True):
                 yield t, batch
 
-    return newfunc
+    return  _wrap_with_parametric_batch
 
 
 # List of most obscure filenames which might or not be supported by different
 # filesystems across different OSs.  Start with the most obscure
 OBSCURE_PREFIX = os.getenv('DATALAD_TESTS_OBSCURE_PREFIX', '')
-OBSCURE_FILENAMES = (
-    u" \"';a&b/&c `| ",  # shouldn't be supported anywhere I guess due to /
-    u" \"';a&b&c `| ",
-    u" \"';abc `| ",
-    u" \"';abc | ",
-    u" \"';abc ",
-    u" ;abc ",
-    u" ;abc",
-    u" ab c ",
-    u" ab c",
-    u"ac",
-    u" ab .datc ",
-    u"ab .datc ",  # they all should at least support spaces and dots
-)
+# Those will be tried to be added to the base name if filesystem allows
+OBSCURE_FILENAME_PARTS = ['/', '|', ';', '&',
+                          '%b5',
+                          ' ']
 UNICODE_FILENAME = u"ΔЙקم๗あ"
+
 # OSX is exciting -- some I guess FS might be encoding differently from decoding
 # so Й might get recoded
 # (ref: https://github.com/datalad/datalad/pull/1921#issuecomment-385809366)
@@ -1425,35 +1454,66 @@ if sys.getfilesystemencoding().lower() == 'utf-8':
     if on_windows:
         # TODO: really figure out unicode handling on windows
         UNICODE_FILENAME = ''
-    # Prepend the list with unicode names first
-    OBSCURE_FILENAMES = tuple(
-        f.replace(u'c', u'c' + UNICODE_FILENAME) for f in OBSCURE_FILENAMES
-    ) + OBSCURE_FILENAMES
+    if UNICODE_FILENAME:
+        OBSCURE_FILENAME_PARTS.append(UNICODE_FILENAME)
+# simple extension to finish it up
+OBSCURE_FILENAME_PARTS.append('.datc')
 
 
 @with_tempfile(mkdir=True)
-def get_most_obscure_supported_name(tdir):
+def get_most_obscure_supported_name(tdir, return_candidates=False):
     """Return the most obscure filename that the filesystem would support under TEMPDIR
 
+    Parameters
+    ----------
+    return_candidates: bool, optional
+      if True, return a tuple of (good, candidates) where candidates are "partially"
+      sorted from trickiest considered
     TODO: we might want to use it as a function where we would provide tdir
     """
-    for filename in OBSCURE_FILENAMES:
-        filename = OBSCURE_PREFIX + filename
-        if on_windows and filename.rstrip() != filename:
-            continue
+    # we need separate good_base so we do not breed leading/trailing spaces
+    initial = good = 'a'  # everyone should support that!
+    system = platform.system()
+
+    OBSCURE_FILENAMES = []
+    def good_filename(filename):
+        OBSCURE_FILENAMES.append(candidate)
         try:
             with open(opj(tdir, filename), 'w') as f:
                 f.write("TEST LOAD")
-            return filename  # it will get removed as a part of wiping up the directory
+            return True
         except:
             lgr.debug("Filename %r is not supported on %s under %s",
-                      filename, platform.system(), tdir)
-            pass
-    raise RuntimeError("Could not create any of the files under %s among %s"
+                      filename, system, tdir)
+            return False
+
+    # incrementally build up the most obscure filename from parts
+    for part in OBSCURE_FILENAME_PARTS:
+        candidate = good + part
+        if good_filename(OBSCURE_PREFIX + candidate):
+            good = candidate
+
+    # now we will compose some candidates with trailing spaces - trickier ones first
+    # so we break the loop as soon as we get it
+    for candidate in [' ' + good + ' ', ' ' + good, good + ' ', good]:
+        candidate = OBSCURE_PREFIX + candidate
+        if on_windows and candidate.rstrip() != candidate:
+            continue
+        if good_filename(candidate):
+            good = candidate
+            break
+
+    if good == initial:
+        raise RuntimeError("Could not create any of the files under %s among %s"
                        % (tdir, OBSCURE_FILENAMES))
+    lgr.debug("Tested %d obscure filename candidates. The winner: %r", len(OBSCURE_FILENAMES), good)
+    if return_candidates:
+        return good, OBSCURE_FILENAMES[::-1]
+    else:
+        return good
 
 
-OBSCURE_FILENAME = get_most_obscure_supported_name()
+OBSCURE_FILENAME, OBSCURE_FILENAMES = get_most_obscure_supported_name(return_candidates=True)
 
 
 @optional_args
@@ -1461,7 +1521,7 @@ def with_testsui(t, responses=None, interactive=True):
     """Switch main UI to be 'tests' UI and possibly provide answers to be used"""
 
     @wraps(t)
-    def newfunc(*args, **kwargs):
+    def  _wrap_with_testsui(*args, **kwargs):
         from datalad.ui import ui
         old_backend = ui.backend
         try:
@@ -1479,7 +1539,7 @@ def with_testsui(t, responses=None, interactive=True):
     if not interactive and responses is not None:
         raise ValueError("Non-interactive UI cannot provide responses")
 
-    return newfunc
+    return  _wrap_with_testsui
 
 with_testsui.__test__ = False
 
@@ -1487,7 +1547,7 @@ with_testsui.__test__ = False
 def assert_no_errors_logged(func, skip_re=None):
     """Decorator around function to assert that no errors logged during its execution"""
     @wraps(func)
-    def new_func(*args, **kwargs):
+    def  _wrap_assert_no_errors_logged(*args, **kwargs):
         with swallow_logs(new_level=logging.ERROR) as cml:
             out = func(*args, **kwargs)
             if cml.out:
@@ -1498,7 +1558,7 @@ def assert_no_errors_logged(func, skip_re=None):
                     )
         return out
 
-    return new_func
+    return  _wrap_assert_no_errors_logged
 
 
 def get_mtimes_and_digests(target_path):
@@ -1819,10 +1879,12 @@ def get_deeply_nested_structure(path):
     return ds
 
 
-def has_symlink_capability():
+@with_tempfile
+@with_tempfile
+def has_symlink_capability(p1, p2):
 
-    path = ut.Path(tempfile.mktemp())
-    target = ut.Path(tempfile.mktemp())
+    path = ut.Path(p1)
+    target = ut.Path(p2)
     return utils.check_symlink_capability(path, target)
 
 
@@ -1834,11 +1896,54 @@ def skip_wo_symlink_capability(func):
     """
     @wraps(func)
     @attr('skip_wo_symlink_capability')
-    def newfunc(*args, **kwargs):
+    def  _wrap_skip_wo_symlink_capability(*args, **kwargs):
         if not has_symlink_capability():
             raise SkipTest("no symlink capabilities")
         return func(*args, **kwargs)
-    return newfunc
+    return  _wrap_skip_wo_symlink_capability
+
+
+def get_ssh_port(host):
+    """Get port of `host` in ssh_config.
+
+    Our tests depend on the host being defined in ssh_config, including its
+    port. This method can be used by tests that want to check handling of an
+    explicitly specified
+
+    Note that if `host` does not match a host in ssh_config, the default value
+    of 22 is returned.
+
+    Parameters
+    ----------
+    host : str
+
+    Returns
+    -------
+    port (int)
+
+    Raises
+    ------
+    SkipTest if port cannot be found.
+    """
+    out = ''
+    try:
+        out, err = Runner()(["ssh", "-G", host])
+    except Exception as exc:
+        err = str(exc)
+
+    port = None
+    for line in out.splitlines():
+        if line.startswith("port "):
+            try:
+                port = int(line.split()[1])
+            except Exception as exc:
+                err = str(exc)
+            break
+
+    if port is None:
+        raise SkipTest("port for {} could not be determined: {}"
+                       .format(host, err))
+    return port
 
 
 #
@@ -1849,8 +1954,7 @@ def skip_wo_symlink_capability(func):
 def patch_config(vars):
     """Patch our config with custom settings. Returns mock.patch cm
     """
-    from datalad import cfg
-    return patch.dict(cfg._store, vars)
+    return patch.dict(dl_cfg._store, vars)
 
 
 @contextmanager
@@ -1901,25 +2005,33 @@ from nose.plugins.attrib import attr
 def integration(f):
     """Mark test as an "integration" test which generally is not needed to be run
     
-    Generally tend to be slower
+    Generally tend to be slower.
+    Should be used in combination with @slow and @turtle if that is the case.
     """
     return attr('integration')(f)
 
 
 def slow(f):
     """Mark test as a slow, although not necessarily integration or usecase test
+
+    Rule of thumb cut-off to mark as slow is 10 sec
     """
     return attr('slow')(f)
 
 
 def turtle(f):
     """Mark test as very slow, meaning to not run it on Travis due to its
-    time limit"""
+    time limit
+
+    Rule of thumb cut-off to mark as turtle is 2 minutes
+    """
     return attr('turtle')(f)
 
 
 def usecase(f):
     """Mark test as a usecase user ran into and which (typically) caused bug report
     to be filed/troubleshooted
+
+    Should be used in combination with @slow and @turtle if slow.
     """
     return attr('usecase')(f)
